//*********************************************************************************************
// NMEA server for transmitting engine and shaft speed via TCP connection
// (C) Norbert Walter 2018
//
// Motordiagnose.ino
//
// Tested and compiled with Arduino IDE 1.8.5
//
// The server use following NMEA centeces:
//
// For engine speed:        $ERRPM,E,0,x.0,45.0,A*hh
// For shaft speed:         $ERRPM,S,0,x.0,45.0,A*hh
// Sea water flow:          $PESWF,L,0,x.0,A*hh         (Custom sentence)
// Cooling water flow:      $PECWF,L,0,x.0,A*hh         (Custom sentence)
// Sea water temp:          $PESWT,C,0,x.x,A*hh         (Custom sentence)
// Sea water temp:          $WIMTW,x.x,C*hh
// Cooling water temp:      $PECWT,C,0,x.x,A*hh         (Custom sentence)
// Engine room temp:        $PEERT,C,0,x.x,A*hh         (Custom sentence)
// Engine live time:        $PELTM,H,0,x.x,A*hh         (Custom sentence)
// 
// RPM - Revolutions Per Minute
//
//  0     1 2 3   4   5 6
//  |     | | |   |   | |
// $--RPM,a,x,x.x,x.x,A*hh<CR><LF>
//
// Field Number: 
//  0) Type of telegram ER = Engine room
//  1) Sourse, S = Shaft, E = Engine
//  2) Engine or shaft number
//  3) Speed, Revolutions per minute
//  4) Propeller pitch, % of maximum, "-" means astern
//  5) Status, A means data is valid
//  6) Checksum
//
// One client can only connecting the server. Each other client has no connection until
// the first client is connected. The NMEA server is reconnectable. The propeller pitch is
// fix at 45 % setted. The engine and shaft numer is fix at 0 setted. Allways send valid data
// and set status on A. Per send period sending two telegrams, one with engine speed and another
// with shaft speed. The NMEA server reduce the telegram frequence when the engine and shaft
// speed is zero (engine is off).
//*********************************************************************************************

// Includes
#include <SPI.h>          // SPI interface for SDCard
#include <SD.h>           // SDCard library
#include <ESP8266WiFi.h>  // WiFi library with TCP server and client
#include <Ticker.h>       // Timer library
#include <EEPROM.h>       // EEPROM library
#include <WString.h>      // Needs for structures
#include <OneWire.h>
#include <DallasTemperature.h>
#include "Configuration.h"// Setup data structure in header file
#include "Definitions.h"  // Local definitions in additional file
#include "FunctionsLib.h" // Local functions declaration in additional file

// Declarations
int value;                // Value from first byte in EEPROM
int empty;                // If EEPROM empty without configutation then set to 1 otherwise 0
configData defconf;       // Definition of default configuration data
configData oldconf;       // Configuration stucture for old config data in EEPROM
configData newconf;       // Configuration stucture for new config data in EEPROM
configData actconf;       // Overload with old EEPROM configuration by start. It is necessarry for port and serspeed

String HexCheckSum;
String NMEAEngineSpeed;
String NMEAShaftSpeed;
String NMEASeaWaterFlow;
String NMEACoolingWaterFlow;
String NMEASeaWaterTemp;
String NMEACoolingWaterTemp;
String NMEAEngineRoomTemp;
String NMEAEngineLiveTime;
String SendEngineSpeed;
String SendShaftSpeed;
String SendSeaWaterFlow;
String SendCoolingWaterFlow;
String SendSeaWaterTemp;
String SendCoolingWaterTemp;
String SendEngineRoomTemp;
String SendEngineLiveTime;

Ticker Timer;             // Declare timer for counter
Ticker Timer2;            // Declare timer for live time counter
WiFiServer server(port);  // Declare WiFi HTTP server with specified port

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature DS18B20(&oneWire);
 
void setup() {
  
  // ***********************************************************************************************
  // ************ Attantion!! **********************************************************************
  // ***********************************************************************************************
  
  // For using a new Wemos D1 mini is no configuration in the EEPROM.
  // In this case you must write a defaut configuration in the EEPROM. Otherwise hanging the Wemos.
  
  // saveEEPROMConfig(defconf);   // Write default configutation
  // saveEEPROMLiveTime(0);       // Reset and write live time counter

  // Use the same procedure when insert or delete any value rows in Configuration.h
  // Delete also the SETTINGS.TXT file on SDCard and rebooting the Wemos
  
  // ***********************************************************************************************

  // Read the first byte from the EEPROM
  EEPROM.begin(sizeEEPROM);
  value = EEPROM.read(cfgStart);
  EEPROM.end(); 
  // If the fist Byte not identical to the first value in the default configuration then saving a default configuration in EEPROM.
  // Means if the EEPROM empty then saving a default configuration.
  if(value == defconf.valid){
    empty = 0;                  // Marker for configuration is present
  }
  else{
    saveEEPROMConfig(defconf);
    empty = 1;                  // Marker for configuration is missing
  }

  // Loading EEPROM configuration
  actconf = loadEEPROMConfig(); // Overload with old EEPROM configuration by start. It is necessarry for serspeed
  debug = actconf.debug;        // Overload with old EEPROM configuration by start

  // Loading the old live time counter
  liveTime = loadEEPROMLiveTime();
  // If offsetLiveTimeCounter = -1 then reset the live time counter
  if(actconf.offsetLiveTimeCounter == int(-1)){
    liveTime = 0;
    saveEEPROMLiveTime(liveTime);
  }
  
  // Pin Settings
  pinMode(ledPin, OUTPUT);          // LED Pin output
  pinMode(ssPin, OUTPUT);           // SS os CS for SDCard
  digitalWrite(ledPin, LOW);        // Switch LED on (inverted!)
  
  Serial.begin(actconf.serspeed);   // Start serial communication
  delay(10);

//  DebugPrint(3, "First EEPROM byte: ");
//  DebugPrintln(3, value);

  // ESP8266 Information Data
  DebugPrintln(3, "NMEA Data Server V1.0 (C) Norbert Walter");
  DebugPrintln(3, "****************************************");
  DebugPrintln(3, "");
  DebugPrintln(3, "Modul Type: Wimo D1 mini");
  DebugPrint(3, "SDK-Version: ");
  DebugPrintln(3, ESP.getSdkVersion());
  DebugPrint(3, "ESP8266 Chip-ID: ");
  DebugPrintln(3, ESP.getChipId());
  DebugPrint(3, "ESP8266 Speed [MHz]: ");  
  DebugPrintln(3, ESP.getCpuFreqMHz());
  DebugPrint(3, "Free Heap Size [Bytes]: ");
  DebugPrintln(3, ESP.getFreeHeap());
  DebugPrintln(3, "");
  DebugPrint(3, "Engine ID: ");
  DebugPrintln(3, actconf.engineID);
  DebugPrint(3, "Offset Value Live Time Counter: ");
  DebugPrintln(3, actconf.offsetLiveTimeCounter);
  DebugPrint(3, "Live Time Counter: ");
  DebugPrintln(3, actconf.liveTimeCounter);
  DebugPrintln(3, "Sensor Type: Engine Speed");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrintln(3, INT_PIN1);
  DebugPrintln(3, "Value Range [rpm]: 0...8000");
  DebugPrintln(3, "Sensor Type: Shaft Speed");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrintln(3, INT_PIN2);
  DebugPrintln(3, "Value Range [rpm]: 0...8000");
  DebugPrintln(3, "Sensor Type: Sea Water Flow");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrintln(3, INT_PIN3);
  DebugPrintln(3, "Value Range [l/min]: 0...30");
  DebugPrintln(3, "Sensor Type: Sea Water Temp 1Wire");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrint(3, ONE_WIRE_BUS);
  DebugPrint(3, " engineID: ");
  DebugPrintln(3, actconf.seaWaterTempID);
  DebugPrintln(3, "Value Range [°C]: -55...125");
  DebugPrintln(3, "Sensor Type: Cooling Water Flow");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrintln(3, INT_PIN4);
  DebugPrintln(3, "Value Range [l/min]: 0...30");
  DebugPrintln(3, "Sensor Type: Cooling Water Temp 1Wire");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrint(3, ONE_WIRE_BUS);
  DebugPrint(3, " engineID: ");
  DebugPrintln(3, actconf.coolingWaterTempID);
  DebugPrintln(3, "Value Range [°C]: -55...125");
  DebugPrintln(3, "Sensor Type: Engine Room Temp 1Wire");
  DebugPrint(3, "Input Pin: GPIO ");
  DebugPrint(3, ONE_WIRE_BUS);
  DebugPrint(3, " engineID: ");
  DebugPrintln(3, actconf.engineRoomTempID);
  DebugPrintln(3, "Value Range [°C]: -55...125");
  DebugPrint(3, "Send Period [ms]: ");
  DebugPrintln(3, SendPeriod);
  DebugPrintln(3, "");

  // Debug info for initialize the EEPROM
  if(empty == 1){
    DebugPrintln(3, "EEPROM config missing, initialization done");
  }
  else{
    DebugPrintln(3, "EEPROM config present");
  }
//#########################
/*  
  // Read and write SDCard configuration data
  if (!SD.begin(ssPin)) {
    DebugPrintln(3, "SDCard initialization failed!");
    DebugPrintln(3, "SDCard not present");
  }
  else {
    DebugPrintln(3, "SDCard initialization done");
    DebugPrintln(3, "SDCard present");
    // print the type and size of card
    Sd2Card card;
    SdVolume volume;
    card.init(SPI_HALF_SPEED, ssPin);
    DebugPrint(3, "Card type: ");
    switch (card.type()) {
      case SD_CARD_TYPE_SD1:
        DebugPrintln(3, "SD1");
        break;
      case SD_CARD_TYPE_SD2:
        DebugPrintln(3, "SD2");
        break;
      case SD_CARD_TYPE_SDHC:
        DebugPrintln(3, "SDHC");
        break;
      default:
        DebugPrintln(3, "Unknown");
    }
    // Try to open the 'volume'/'partition' - it should be FAT16 or FAT32
    if (!volume.init(card)) {
      DebugPrintln(3, "Could not find FAT16/FAT32 partition!");
      DebugPrintln(3, "Make sure you've formatted the card.");
      return;
    }
    else{
      // Print the type and size of the first FAT-type volume
      uint32_t volumesize;
      DebugPrint(3, "Volume type is FAT");
      int typeNumber = volume.fatType();
//      DebugPrintln(3, volume.fatType(), DEC);
      DebugPrintln(3, typeNumber);
      volumesize = volume.blocksPerCluster();    // Clusters are collections of blocks
      volumesize *= volume.clusterCount();       // We'll have a lot of clusters
      volumesize *= 512;                         // SD card blocks are always 512 bytes
      volumesize /= 1024;                        // Size in kB
      volumesize /= 1024;                        // Size in MB
      DebugPrint(3, "Volume size [Mbytes]: ");
      DebugPrintln(3, volumesize);
      DebugPrintln(3, "");
      
      // Is config file not present
      if (!SD.open(configfile)){
        DebugPrintln(3, "No config file found");
        DebugPrintln(3, "Loading default config");
        newconf = defconf;        // Load default configuration
        DebugPrintln(3, "Writing default config in EEPROM");
        saveEEPROMConfig(newconf);// Write new configuration in EEPROM
        DebugPrintln(3, "Creating new config file on SDCard");
        writeSDConfig(configfile);
      }
      else{
        DebugPrintln(3, "Reading existing config file from SDCard");
        newconf = readSDConfig(configfile);
        DebugPrintln(3, "Comparing config from EEPROM with SDCard");
        oldconf = loadEEPROMConfig();
//        String test = "1234";
//        test.toCharArray(oldconf.password, 31);
        //When structures newconf and oldconf equal
        if (memcmp(&newconf, &oldconf, sizeof newconf) == 0){
          DebugPrintln(3, "Configurations are equal");
        }
        else{
          DebugPrintln(3, "Configurations not equal");
          DebugPrintln(3, "Writing config from SDCard in EEPROM");
          saveEEPROMConfig(newconf);// Write new configuration in EEPROM
        }
      }
    }
  }
*/
//###############################################################  

  // Loading EEPROM config
  DebugPrintln(3, "Loading actual EEPROM config");
  actconf = loadEEPROMConfig();
  DebugPrintln(3, "");

  //*************************************************
  // Starts the ínterrupts allways after SDCard using
  // because the SDCard pins are duble used from
  // the digital sensors
  //*************************************************
  // Pin settings for interrupt
  pinMode(INT_PIN1, INPUT_PULLUP);  // Interrupt input 1
  pinMode(INT_PIN2, INPUT_PULLUP);  // Interrupt input 2
  pinMode(INT_PIN3, INPUT_PULLUP);  // Interrupt input 3
  pinMode(INT_PIN4, INPUT_PULLUP);  // Interrupt input 4
  
  Timer.attach(1, CalcValues);      // Start Timer all 1s value calculation
  Timer2.attach(180, LiveTime);     // Start Timer all 180s or 3min live time counter
                                    // Max EEPROM write cycle is 100.000
                                    // Livetime from EEPROM ends after 5000h engine time
  // Start interrupts
  attachInterrupt(INT_PIN1, interruptRoutine1, FALLING); // Start interrupt for engine speed
  attachInterrupt(INT_PIN2, interruptRoutine2, FALLING); // Start Interrupt for shaft speed
  attachInterrupt(INT_PIN3, interruptRoutine3, FALLING); // Start interrupt for sea water flow
  attachInterrupt(INT_PIN4, interruptRoutine4, FALLING); // Start Interrupt for cooling water flow
  //**************************************************
 
  // Connect to WiFi network
  DebugPrint(3, "Connecting to ");
  DebugPrintln(3, actconf.ssid);
 
  // Wait until is connected
  WiFi.begin(actconf.ssid, actconf.password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    DebugPrint(3, ".");
  }
  DebugPrintln(3, "");
  DebugPrintln(3, "WiFi connected");
  digitalWrite(ledPin, HIGH);         // Switch LED off
  delay(100);
 
  // Start the server
  server.begin();
  DebugPrint(3, "NMEA-Server started at port: ");
  DebugPrintln(3, port);
  // Print the IP address
  DebugPrintln(3, "For testing use:");
  DebugPrint(3, "telnet ");
  DebugPrint(3, WiFi.localIP());
  DebugPrint(3, " ");
  DebugPrintln(3, port);
}
 
void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  int i = 0;
  // While client is connected
  while (client.connected() && !client.available()) {
  if (i == 0){
    DebugPrintln(3, "Client connected");
    DebugPrintln(3, "");
  }
  i = i + 1;
  // Send data
  DebugPrint(3, "Send package:");
  DebugPrintln(3, i);
  
  // Live time counter routine
  if(saveLiveTime == 1){
    liveTime = loadEEPROMLiveTime();
    liveTime += 0.05;     // wake up all 3 min (0.05 h)
    saveEEPROMLiveTime(liveTime);
    saveLiveTime = 0;
  }
  // If reset for live time counter then using without offset
  if(actconf.offsetLiveTimeCounter == int(-1)){
    sumLiveTime = liveTime;
  }
  else{
    sumLiveTime = liveTime + float(actconf.offsetLiveTimeCounter);
  }

  // *****************************
  
  if (actconf.engineSpeed == 1){  
    DebugPrint(3, "Engine Speed [1/min]:");
    DebugPrintln(3, result[0]);
 
    // Create NMEA string for engine speed $ERRPM,E,0,x.0,45.0,A*hh<CR><LF>
    NMEAEngineSpeed = "ERRPM,E,";
    NMEAEngineSpeed += String(actconf.engineID);
    NMEAEngineSpeed += ",";
    NMEAEngineSpeed += String(result[0]);
    NMEAEngineSpeed += ".0,45.0,A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEAEngineSpeed), HEX);
    // Build complete NMEA string
    SendEngineSpeed = "$" + NMEAEngineSpeed;
    SendEngineSpeed += "*";
    SendEngineSpeed += HexCheckSum;
    Serial.println(SendEngineSpeed);
    client.println(SendEngineSpeed);
  }

  if (actconf.shaftSpeed == 1){ 
    DebugPrint(3, "Shaft Speed [1/min]:");
    DebugPrintln(3, result[1]);
  
    // Create NMEA string for shaft speed $ERRPM,S,0,x.0,45.0,A*hh<CR><LF>
    NMEAShaftSpeed = "ERRPM,S,";
    NMEAShaftSpeed += String(actconf.engineID);
    NMEAShaftSpeed += ",";
    NMEAShaftSpeed += String(result[1]);
    NMEAShaftSpeed += ".0,45.0,A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEAShaftSpeed), HEX);
    // Build complete NMEA string
    SendShaftSpeed = "$" + NMEAShaftSpeed;
    SendShaftSpeed += "*";
    SendShaftSpeed += HexCheckSum;
    Serial.println(SendShaftSpeed);
    client.println(SendShaftSpeed);
  }

  if (actconf.seaWaterFlow == 1){ 
    DebugPrint(3, "Sea Water Flow [l/min]:");
    DebugPrintln(3, result[2]);
  
    // Create NMEA string for sea water flow $PESWF,L,0,x.0,A*hh<CR><LF>
    NMEASeaWaterFlow = "PESWF,L,";
    NMEASeaWaterFlow += String(actconf.engineID);
    NMEASeaWaterFlow += ",";
    NMEASeaWaterFlow += String(result[2]);
    NMEASeaWaterFlow += ".0,A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEASeaWaterFlow), HEX);
    // Build complete NMEA string
    SendSeaWaterFlow = "$" + NMEASeaWaterFlow;
    SendSeaWaterFlow += "*";
    SendSeaWaterFlow += HexCheckSum;
    Serial.println(SendSeaWaterFlow);
    client.println(SendSeaWaterFlow);
  }

  if (actconf.coolingWaterFlow == 1){ 
    DebugPrint(3, "Cooling Water Flow [l/min]:");
    DebugPrintln(3, result[3]);
  
    // Create NMEA string for cooling water flow $PECWF,L,0,x.0,A*hh<CR><LF>
    NMEACoolingWaterFlow = "PECWF,L,";
    NMEACoolingWaterFlow += String(actconf.engineID);
    NMEACoolingWaterFlow += ",";
    NMEACoolingWaterFlow += String(result[3]);
    NMEACoolingWaterFlow += ".0,A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEACoolingWaterFlow), HEX);
    // Build complete NMEA string
    SendCoolingWaterFlow = "$" + NMEACoolingWaterFlow;
    SendCoolingWaterFlow += "*";
    SendCoolingWaterFlow += HexCheckSum;
    Serial.println(SendCoolingWaterFlow);
    client.println(SendCoolingWaterFlow);
  }

  if (actconf.seaWaterTemp == 1){
    DS18B20.requestTemperatures(); 
    String tempc;
    String tempf;
    tempc = String(DS18B20.getTempCByIndex(actconf.seaWaterTempID));
    tempf = String(DS18B20.getTempFByIndex(actconf.seaWaterTempID));
    
    // Create NMEA string for sea water temperature $PESWT,C,0,x.x,A*hh<CR><LF>
    if(String(actconf.tempUnit) == "C"){
      NMEASeaWaterTemp = "PESWT,C,";
      NMEASeaWaterTemp += String(actconf.engineID);
      NMEASeaWaterTemp += ",";
      NMEASeaWaterTemp += String(tempc);
      DebugPrint(3, "Sea Water Temp [°C]:");
      DebugPrintln(3, tempc);
    }
    else{
      NMEASeaWaterTemp = "PESWT,F,";
      NMEASeaWaterTemp += String(actconf.engineID);
      NMEASeaWaterTemp += ",";
      NMEASeaWaterTemp += String(tempf);
      DebugPrint(3, "Sea Water Temp [°F]:");
      DebugPrintln(3, tempf);
    }
    NMEASeaWaterTemp += ",A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEASeaWaterTemp), HEX);
    // Build complete NMEA string
    SendSeaWaterTemp = "$" + NMEASeaWaterTemp;
    SendSeaWaterTemp += "*";
    SendSeaWaterTemp += HexCheckSum;
    Serial.println(SendSeaWaterTemp);
    client.println(SendSeaWaterTemp);
  }

  if (actconf.seaWaterTemp == 1){
    DS18B20.requestTemperatures(); 
    String tempc;
    String tempf;
    tempc = String(DS18B20.getTempCByIndex(actconf.seaWaterTempID));
    tempf = String(DS18B20.getTempFByIndex(actconf.seaWaterTempID));
    
    // Create NMEA string for sea water temperature $WIMTW,x.x,C*hh<CR><LF>
    if(String(actconf.tempUnit) == "C"){
      NMEASeaWaterTemp = "WIMTW," + String(tempc);
      NMEASeaWaterTemp += ",C,";
      DebugPrint(3, "Sea Water Temp [°C]:");
      DebugPrintln(3, tempc);
    }
    else{
      NMEASeaWaterTemp = "WIMTW," + String(tempf);
      NMEASeaWaterTemp += ",F,";
      DebugPrint(3, "Sea Water Temp [°F]:");
      DebugPrintln(3, tempf);
    }
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEASeaWaterTemp), HEX);
    // Build complete NMEA string
    SendSeaWaterTemp = "$" + NMEASeaWaterTemp;
    SendSeaWaterTemp += "*";
    SendSeaWaterTemp += HexCheckSum;
    Serial.println(SendSeaWaterTemp);
    client.println(SendSeaWaterTemp);
  }

  if (actconf.coolingWaterTemp == 1){
    DS18B20.requestTemperatures(); 
    String tempc;
    String tempf;
    tempc = String(DS18B20.getTempCByIndex(actconf.coolingWaterTempID));
    tempf = String(DS18B20.getTempFByIndex(actconf.coolingWaterTempID));
    
    // Create NMEA string for cooling water temperature $PECWT,C,0,x.x,A*hh<CR><LF>
    if(String(actconf.tempUnit) == "C"){
      NMEACoolingWaterTemp = "PECWT,C,";
      NMEACoolingWaterTemp += String(actconf.engineID);
      NMEACoolingWaterTemp += ",";
      NMEACoolingWaterTemp += String(tempc);
      DebugPrint(3, "Cooling Water Temp [°C]:");
      DebugPrintln(3, tempc);
    }
    else{
      NMEACoolingWaterTemp = "PECWT,F,";
      NMEACoolingWaterTemp += String(actconf.engineID);
      NMEACoolingWaterTemp += ",";
      NMEACoolingWaterTemp += String(tempf);
      DebugPrint(3, "Cooling Water Temp [°F]:");
      DebugPrintln(3, tempf);
    }
    NMEACoolingWaterTemp += ",A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEACoolingWaterTemp), HEX);
    // Build complete NMEA string
    SendCoolingWaterTemp = "$" + NMEACoolingWaterTemp;
    SendCoolingWaterTemp += "*";
    SendCoolingWaterTemp += HexCheckSum;
    Serial.println(SendCoolingWaterTemp);
    client.println(SendCoolingWaterTemp);
  }

  if (actconf.engineRoomTemp == 1){
    DS18B20.requestTemperatures(); 
    String tempc;
    String tempf;
    tempc = String(DS18B20.getTempCByIndex(actconf.engineRoomTempID));
    tempf = String(DS18B20.getTempFByIndex(actconf.engineRoomTempID));
    
    // Create NMEA string for engine room temperature $PEERT,C,0,x.x,A*hh<CR><LF>
    if(String(actconf.tempUnit) == "C"){
      NMEAEngineRoomTemp = "PEERT,C,";
      NMEAEngineRoomTemp += String(actconf.engineID);
      NMEAEngineRoomTemp += ",";
      NMEAEngineRoomTemp += String(tempc);
      DebugPrint(3, "Engine Room Temp [°C]:");
      DebugPrintln(3, tempc);
    }
    else{
      NMEAEngineRoomTemp = "PEERT,F,";
      NMEAEngineRoomTemp += String(actconf.engineID);
      NMEAEngineRoomTemp += ",";
      NMEAEngineRoomTemp += String(tempf);
      DebugPrint(3, "Engine Room Temp [°F]:");
      DebugPrintln(3, tempf);
    }
    NMEAEngineRoomTemp += ",A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEAEngineRoomTemp), HEX);
    // Build complete NMEA string
    SendEngineRoomTemp = "$" + NMEAEngineRoomTemp;
    SendEngineRoomTemp += "*";
    SendEngineRoomTemp += HexCheckSum;
    Serial.println(SendEngineRoomTemp);
    client.println(SendEngineRoomTemp);
  }

  if (actconf.liveTimeCounter == 1){
    // Create NMEA string for engine live time $PELTM,H,0,x.x,A*hh<CR><LF>
    DebugPrint(3, "Engine Live Time [h]:");
    DebugPrintln(3, sumLiveTime);
    NMEAEngineLiveTime = "PELTM,H,";
    NMEAEngineLiveTime += String(actconf.engineID);
    NMEAEngineLiveTime += ",";
    NMEAEngineLiveTime += String(sumLiveTime);
    NMEAEngineLiveTime += ",A";
    // Build CheckSum
    HexCheckSum = String(CheckSum(NMEAEngineLiveTime), HEX);
    // Build complete NMEA string
    SendEngineLiveTime = "$" + NMEAEngineLiveTime;
    SendEngineLiveTime += "*";
    SendEngineLiveTime += HexCheckSum;
    Serial.println(SendEngineLiveTime);
    client.println(SendEngineLiveTime);
  }
  
  //Blinking LED for transmitting data (Low activ)
  digitalWrite(ledPin, LOW);
  delay(10);
  digitalWrite(ledPin, HIGH);

  //Reduce the telegram frequence when engine and shaft speed is zero
  if (result[0] == 0 && result[1] == 0 && result[2] == 0 && result[3] == 0){
  delay(RedSendPeriod - 10);  //subsract 10ms because LED was on
  }
  else {
    delay(SendPeriod -10);    //subsract 10ms because LED was on
  }
  DebugPrintln(3, "");  
  }
}
